@@ -18,4 +18,12 @@
 #  along with quality-result-gui. If not, see <https://www.gnu.org/licenses/>.
 
 
-__version__ = "0.0.4"
+__version__ = "1.0.0"
+
+
+from enum import Enum
+
+
+class SelectionType(Enum):
+    RightClick = 1
+    Other = 2
